@@ -8,8 +8,8 @@ import bisect
 from datetime import datetime, timedelta
 from microt_compliance_matrix import utils
 from microt_compliance_matrix.utils.get_time_diff import *
-
 from microt_compliance_matrix.utils.uncompress_file import *
+from microt_compliance_matrix.utils.get_haversine_distance import *
 
 target_file_pattern = 'phone_GPS_clean*.csv.zip'
 
@@ -44,7 +44,6 @@ def validate_dates_before_after(intermediate_participant_path, date_in_study):
         if len(one_day_after_log_paths) != 0:
             validated_date_list.append(one_date_after)
 
-
     return validated_date_list
 
 # def combine_intermediate_file(intermediate_participant_path, date_in_study, decryption_password):
@@ -120,7 +119,7 @@ def combine_intermediate_file(intermediate_participant_path, date_in_study, decr
             df_day = pd.read_csv(unzip_file_path)
         except pd.errors.EmptyDataError:
             print("pandas.errors.EmptyDataError (location) : " + unzip_file_path)
-            quit()
+            return df_logs_combined
 
         if df_day.shape[0] > 0:
             df_day['Participant_ID'] = [participant_id] * df_day.shape[0]
@@ -143,20 +142,36 @@ def find_closest_time(prompt_time, subset_time_list):
     closet_time = min(subset_time_list[max(0, i - 1): i + 2], key=lambda t: abs(prompt_time - t))
     return closet_time
 
-def match_feature(prompt_local_datetime_series, df_logs_combined):
+def get_distance_from_home(location, info_dict):
+    distance_from_home = "bboxNF"  # this means the lsf.json has no bbox for home cluster (i.e., lsf.json recovery error)
+    if type(location) != str:
+        distance_from_home_candidate_list = []
+        home_coords = info_dict["home_loc"]
+        for home_coord in home_coords:
+            d_tmp = haversine(location, home_coord)
+            if distance_from_home == "bboxNF":
+                distance_from_home = d_tmp
+            if d_tmp < distance_from_home:
+                distance_from_home = d_tmp
 
+    return distance_from_home
+
+
+def match_feature(prompt_local_datetime_series, df_logs_combined, info_dict, participant_id):
     print("     --- start matching")
     matched_location_list = []
     matched_time_list = []
+    distance_from_home_list = []
+    p_id_exist_info_dict = True if participant_id in info_dict else False
 
     for idx in prompt_local_datetime_series.index:
         prompt_time = prompt_local_datetime_series[idx]
         # prompt_date = prompt_time.date()
 
-
         if df_logs_combined.shape[0] == 0:
             location = "NF"
             closest_time = "NF"
+            distance_from_home = "NF"  # this means the location file is missing
         else:
             subset_time_list = list(df_logs_combined["LOCATION_TIME_DATETIME"])
 
@@ -174,9 +189,13 @@ def match_feature(prompt_local_datetime_series, df_logs_combined):
 
         matched_time_list.append(closest_time)
         matched_location_list.append(location)
+        if p_id_exist_info_dict:
+            distance_from_home = get_distance_from_home(location, info_dict[participant_id])
+        else:
+            distance_from_home = "JsonNF"  # this means this participant has no recovered lsf.json
+        distance_from_home_list.append(distance_from_home)
 
-
-    return matched_location_list, matched_time_list
+    return matched_location_list, matched_time_list, distance_from_home_list
 
 
 # def reverse_geo_nominatim(lat, long):
@@ -196,7 +215,8 @@ def transform(location_column):
     return location_column
 
 
-def create_column(prompt_local_datetime_series, intermediate_participant_path, date_in_study, decryption_password):
+def create_column(participant_id, prompt_local_datetime_series, info_dict, intermediate_participant_path, date_in_study,
+                  decryption_password):
     print("\n> start generating the feature: Location ")
 
     # Read, parse and combine related intermediate file
@@ -204,16 +224,18 @@ def create_column(prompt_local_datetime_series, intermediate_participant_path, d
 
     if df_logs_combined.shape[0] > 0:
         # Match the combined parsed intermediate file with prompt feature data frame
-        location_column, match_time = match_feature(prompt_local_datetime_series, df_logs_combined)
+        location_column, match_time, DFH_column = match_feature(prompt_local_datetime_series, df_logs_combined,
+                                                                info_dict, participant_id)
     else:
         location_column = ["NF"] * len(prompt_local_datetime_series)
-        match_time= ["NF"] * len(prompt_local_datetime_series)
+        match_time = ["NF"] * len(prompt_local_datetime_series)
+        DFH_column = ["NF"] * len(prompt_local_datetime_series)
 
     # transform feature
     location_column_transformed = transform(location_column)
     print("     --- success")
 
-    return location_column_transformed, match_time
+    return location_column_transformed, match_time, DFH_column
 
 
 if __name__ == "__main__":
